@@ -6,6 +6,7 @@ import acoustid
 import json
 import magic
 import mimetypes
+import os.path
 import tempfile
 import time
 import traceback
@@ -20,17 +21,21 @@ try:
 except ImportError:  # handle test suite import
     from tests.common import ACOUSTID_API_KEY, VERSION, config
 
-    USER_AGENT = f'Ear Tag (Test suite)/{VERSION} (https://gitlab.gnome.org/World/eartag)'
+    USER_AGENT = (
+        f"Ear Tag (Test suite)/{VERSION} (https://gitlab.gnome.org/World/eartag)"
+    )
     TEST_SUITE = True
 else:
     from .config import config
-    USER_AGENT = f'Ear Tag/{VERSION} (https://gitlab.gnome.org/World/eartag)'
+
+    USER_AGENT = f"Ear Tag/{VERSION} (https://gitlab.gnome.org/World/eartag)"
     TEST_SUITE = False
 
 from .utils import simplify_string, simplify_compare, title_case_preserve_uppercase
 
 LAST_REQUEST = 0
 
+
 def make_request(url, raw=False, _recursion=0):
     """Wrapper for urllib.request.Request that handles the setup."""
     global LAST_REQUEST
@@ -71,7 +76,8 @@ def make_request(url, raw=False, _recursion=0):
 
     return data
 
-def build_url(endpoint, id='', **kwargs):
+
+def build_url(endpoint, id="", **kwargs):
     """Builds a MusicBrainz API endpoint URL."""
     args = []
     for argname, argdata in kwargs.items():
@@ -82,6 +88,7 @@ def build_url(endpoint, id='', **kwargs):
         return f'https://musicbrainz.org/ws/2/{endpoint}/{id}?{"&".join(args)}&fmt=json'
     return f'https://musicbrainz.org/ws/2/{endpoint}?{"&".join(args)}&fmt=json'
 
+
 def get_recordings_for_file(file):
     """
     Takes an EartagFile and returns a list of MusicBrainzRecording objects
@@ -91,41 +98,53 @@ def get_recordings_for_file(file):
         raise ValueError("Not enough data for a query; need at least title and artist")
 
     query_data = {
-        'recording': file.title,
-        'artist': file.artist,
-        'release': file.album or '',
+        "recording": file.title,
+        "artist": file.artist,
+        "release": file.album or "",
     }
 
     search_data = make_request(
-        build_url('recording', '', query=' AND '.join(
-            [f'{k}:{v}' for k, v in query_data.items() if v]
-        ))
+        build_url(
+            "recording",
+            "",
+            query=" AND ".join([f"{k}:{v}" for k, v in query_data.items() if v]),
+        )
     )
 
-    if not search_data or 'recordings' not in search_data or not search_data['recordings']:
+    if (
+        not search_data
+        or "recordings" not in search_data
+        or not search_data["recordings"]
+    ):
         # Try to use simplified title/artist:
-        query_data['title'] = simplify_string(file.title)
-        if not query_data['title']:
+        query_data["title"] = simplify_string(file.title)
+        if not query_data["title"]:
             return []
-        query_data['artist'] = simplify_string(file.title)
-        if not query_data['artist']:
+        query_data["artist"] = simplify_string(file.title)
+        if not query_data["artist"]:
             return []
 
         search_data = make_request(
-            build_url('recording', '', query=' AND '.join(
-                [f'{k}:{v}' for k, v in query_data.items() if v]
-            ))
+            build_url(
+                "recording",
+                "",
+                query=" AND ".join([f"{k}:{v}" for k, v in query_data.items() if v]),
+            )
         )
-        if not search_data or 'recordings' not in search_data or not search_data['recordings']:
+        if (
+            not search_data
+            or "recordings" not in search_data
+            or not search_data["recordings"]
+        ):
             return []
 
     ret = []
-    for r in search_data['recordings']:
-        if r['score'] < config['musicbrainz-confidence-treshold']:
+    for r in search_data["recordings"]:
+        if r["score"] < config["musicbrainz-confidence-treshold"]:
             continue
 
         try:
-            rec = MusicBrainzRecording(r['id'], file=file)
+            rec = MusicBrainzRecording(r["id"], file=file)
         except ValueError:
             continue
 
@@ -135,8 +154,8 @@ def get_recordings_for_file(file):
                 if rel.title == file.album or simplify_compare(rel.title, file.album):
                     ret.insert(0, rec)
                     rec.release = rel
-                    rec.available_releases.insert(0,
-                        rec.available_releases.pop(rec.available_releases.index(rel))
+                    rec.available_releases.insert(
+                        0, rec.available_releases.pop(rec.available_releases.index(rel))
                     )
                     _break = True
                     break
@@ -147,8 +166,9 @@ def get_recordings_for_file(file):
 
     return ret
 
+
 class MusicBrainzRecording(GObject.Object):
-    __gtype_name__ = 'MusicBrainzRecording'
+    __gtype_name__ = "MusicBrainzRecording"
 
     SELECT_RELEASE_FIRST = -1
 
@@ -169,12 +189,14 @@ class MusicBrainzRecording(GObject.Object):
     def update_data(self):
         """Updates the internal data struct."""
         self.mb_data = make_request(
-            build_url('recording', self._recording_id,
-                inc=('releases', 'genres', 'artist-credits', 'media', 'release-groups')
+            build_url(
+                "recording",
+                self._recording_id,
+                inc=("releases", "genres", "artist-credits", "media", "release-groups"),
             )
         )
 
-        if not self.mb_data:
+        if not self.mb_data or "title" not in self.mb_data or not self.mb_data["title"]:
             raise ValueError("Could not get recording data")
 
         self.available_releases = self.get_releases()
@@ -183,17 +205,17 @@ class MusicBrainzRecording(GObject.Object):
         # then albums (physical media), then singles, then compilations.
         comps = []
         for rel in self.available_releases.copy():
-            if rel.status != 'official':
+            if rel.status != "official":
                 self.available_releases.remove(rel)
                 comps.append(rel)
                 continue
-            if 'compilation' in rel.group.secondary_types:
+            if "compilation" in rel.group.secondary_types:
                 self.available_releases.remove(rel)
                 comps.append(rel)
                 continue
 
         rels = []
-        for reltype in ('album', 'ep', 'single', 'other'):
+        for reltype in ("album", "ep", "single", "other"):
             checked_ids = []
             for rel in self.available_releases + comps:
                 if rel.group.relgroup_id in checked_ids:
@@ -233,8 +255,8 @@ class MusicBrainzRecording(GObject.Object):
             for rel in self.available_releases:
                 if rel == self.release:
                     continue
-                if rel.cover_tempfiles['thumbnail']:
-                    rel.cover_tempfiles['thumbnail'].close()
+                if rel.cover_tempfiles["thumbnail"]:
+                    rel.cover_tempfiles["thumbnail"].close()
 
     def apply_data_to_file(self, file):
         """
@@ -247,16 +269,30 @@ class MusicBrainzRecording(GObject.Object):
             return False
 
         self.release.update_covers(if_needed=True)
-        for prop in ('title', 'artist', 'album', 'genre', 'albumartist', 'releasedate',
-                'tracknumber', 'totaltracknumber', 'front_cover_path', 'back_cover_path'):
+        for prop in (
+            "title",
+            "artist",
+            "album",
+            "genre",
+            "albumartist",
+            "releasedate",
+            "tracknumber",
+            "totaltracknumber",
+            "front_cover_path",
+            "back_cover_path",
+        ):
             if self.get_property(prop):
                 file.set_property(prop, self.get_property(prop))
 
         file.props.musicbrainz_recordingid = self.recording_id
         file.props.musicbrainz_albumid = self.release.release_id
         file.props.musicbrainz_releasegroupid = self.release.group.relgroup_id
-        file.props.musicbrainz_trackid = self.release.mb_data['media'][0]['tracks'][0]['id']
-        file.props.musicbrainz_artistid = self.mb_data['artist-credit'][0]['artist']['id']
+        file.props.musicbrainz_trackid = self.release.mb_data["media"][0]["tracks"][0][
+            "id"
+        ]
+        file.props.musicbrainz_artistid = self.mb_data["artist-credit"][0]["artist"][
+            "id"
+        ]
 
     @GObject.Property(type=str)
     def recording_id(self):
@@ -274,8 +310,8 @@ class MusicBrainzRecording(GObject.Object):
     def get_releases(self):
         """Returns a list of MusicBrainzRelease object that match the file."""
         releases = []
-        if 'releases' in self.mb_data:
-            for release in self.mb_data['releases']:
+        if "releases" in self.mb_data:
+            for release in self.mb_data["releases"]:
                 releases.append(MusicBrainzRelease(release))
         return releases
 
@@ -291,17 +327,26 @@ class MusicBrainzRecording(GObject.Object):
         self._release = value
 
         # Update everything that depends on the release:
-        for prop in ('album', 'albumartist', 'genre', 'tracknumber', 'totaltracknumber',
-                    'releasedate', 'thumbnail_path', 'front_cover_path', 'back_cover_path'):
+        for prop in (
+            "album",
+            "albumartist",
+            "genre",
+            "tracknumber",
+            "totaltracknumber",
+            "releasedate",
+            "thumbnail_path",
+            "front_cover_path",
+            "back_cover_path",
+        ):
             self.notify(prop)
 
     @GObject.Property(type=str)
     def title(self):
-        return self.mb_data['title']
+        return self.mb_data["title"]
 
     @GObject.Property(type=str)
     def artist(self):
-        return self.mb_data['artist-credit'][0]['name']
+        return self.mb_data["artist-credit"][0]["name"]
 
     @GObject.Property(type=str)
     def album(self):
@@ -313,17 +358,17 @@ class MusicBrainzRecording(GObject.Object):
 
     @GObject.Property(type=str)
     def genre(self):
-        if 'genres' in self.mb_data and self.mb_data['genres']:
-            return title_case_preserve_uppercase(self.mb_data['genres'][0]['name'])
+        if "genres" in self.mb_data and self.mb_data["genres"]:
+            return title_case_preserve_uppercase(self.mb_data["genres"][0]["name"])
         return self.release.genre
 
     @GObject.Property(type=int)
     def tracknumber(self):
-        return int(self.release.mb_data['media'][0]['tracks'][0]['number'])
+        return int(self.release.mb_data["media"][0]["tracks"][0]["number"])
 
     @GObject.Property(type=int)
     def totaltracknumber(self):
-        return int(self.release.mb_data['media'][0]['track-count'])
+        return int(self.release.mb_data["media"][0]["track-count"])
 
     @GObject.Property(type=str)
     def releasedate(self):
@@ -342,7 +387,9 @@ class MusicBrainzRecording(GObject.Object):
         return self.release.back_cover_path
 
     def __str__(self):
-        return f'MusicBrainzRecording {self.recording_id} ({self.title} - {self.artist})'
+        return (
+            f"MusicBrainzRecording {self.recording_id} ({self.title} - {self.artist})"
+        )
 
     def __repr__(self):
         return self.__str__()
@@ -357,7 +404,7 @@ class MusicBrainzRecording(GObject.Object):
 
 
 class MusicBrainzRelease(GObject.Object):
-    __gtype_name__ = 'MusicBrainzRelease'
+    __gtype_name__ = "MusicBrainzRelease"
 
     NEED_UPDATE_COVER = -2
 
@@ -370,16 +417,24 @@ class MusicBrainzRelease(GObject.Object):
         self.mb_data = release_data
         self._from_id = from_id
         self.cover_tempfiles = {
-            'thumbnail': None,
-            'front': MusicBrainzRelease.NEED_UPDATE_COVER,
-            'back': MusicBrainzRelease.NEED_UPDATE_COVER
+            "thumbnail": None,
+            "front": MusicBrainzRelease.NEED_UPDATE_COVER,
+            "back": MusicBrainzRelease.NEED_UPDATE_COVER,
         }
 
         # Get full release data
         if self.release_id not in MusicBrainzRelease.full_data_cache:
             MusicBrainzRelease.full_data_cache[self.release_id] = make_request(
-                build_url('release', self.release_id,
-                    inc=['artist-credits', 'recordings', 'release-groups', 'genres', 'media']
+                build_url(
+                    "release",
+                    self.release_id,
+                    inc=[
+                        "artist-credits",
+                        "recordings",
+                        "release-groups",
+                        "genres",
+                        "media",
+                    ],
                 )
             )
 
@@ -390,15 +445,15 @@ class MusicBrainzRelease(GObject.Object):
         if self.release_id not in MusicBrainzRelease.obj_cache:
             MusicBrainzRelease.obj_cache[self.release_id] = self
 
-        self.group = MusicBrainzReleaseGroup(self.mb_data['release-group'])
+        self.group = MusicBrainzReleaseGroup(self.mb_data["release-group"])
 
         self.update_thumbnail()
 
-    @classmethod
-    def setup_from_id(cls, id):
-        if id in cls.obj_cache:
-            return cls.obj_cache[id]
-        return cls.__init__({}, from_id=id)
+    @staticmethod
+    def setup_from_id(id):
+        if id in MusicBrainzRelease.obj_cache:
+            return MusicBrainzRelease.obj_cache[id]
+        return MusicBrainzRelease(release_data={}, from_id=id)
 
     def dispose(self):
         for temp in self.cover_tempfiles.values():
@@ -409,79 +464,81 @@ class MusicBrainzRelease(GObject.Object):
     def release_id(self):
         if self._from_id:
             return self._from_id
-        return self.mb_data['id']
+        return self.mb_data["id"]
 
     @GObject.Property(type=str)
     def title(self):
-        return self.mb_data['title']
+        return self.mb_data["title"]
 
     @GObject.Property(type=str)
     def artist(self):
-        return self.mb_data['artist-credit'][0]['name']
+        return self.mb_data["artist-credit"][0]["name"]
 
     @GObject.Property(type=str)
     def genre(self):
-        if 'genres' in self.mb_data and self.mb_data['genres']:
-            return title_case_preserve_uppercase(self.mb_data['genres'][0]['name'])
-        return ''
+        if "genres" in self.mb_data and self.mb_data["genres"]:
+            return title_case_preserve_uppercase(self.mb_data["genres"][0]["name"])
+        return ""
 
     @GObject.Property(type=str)
     def releasedate(self):
-        if 'first-release-date' in self.group.mb_data:
-            return self.group.mb_data['first-release-date']
-        if 'date' in self.mb_data:
-            return self.mb_data['date']
-        return ''
+        if "first-release-date" in self.group.mb_data:
+            return self.group.mb_data["first-release-date"]
+        if "date" in self.mb_data:
+            return self.mb_data["date"]
+        return ""
 
     @GObject.Property(type=int)
     def totaltracknumber(self):
-        return int(self.mb_data['media'][0]['track-count'])
+        return int(self.mb_data["media"][0]["track-count"])
 
     @property
     def tracks(self):
-        return self.full_data['media'][0]['tracks']
+        return self.full_data["media"][0]["tracks"]
 
     @GObject.Property(type=str)
     def status(self):
-        return self.mb_data['status'].lower()
+        if "status" in self.mb_data and self.mb_data["status"]:
+            return self.mb_data["status"].lower()
+        return "official"
 
     @GObject.Property(type=str)
     def disambiguation(self):
-        return self.mb_data['disambiguation']
+        return self.mb_data["disambiguation"]
 
     # Covers
 
     @GObject.Property(type=str)
     def thumbnail_path(self):
-        if not self.cover_tempfiles['thumbnail']:
+        if not self.cover_tempfiles["thumbnail"]:
             return self.group.thumbnail_path
-        return self.cover_tempfiles['thumbnail'].name
+        return self.cover_tempfiles["thumbnail"].name
 
     @GObject.Property(type=str)
     def front_cover_path(self):
-        if not self.cover_tempfiles['front']:
+        if not self.cover_tempfiles["front"]:
             return self.group.front_cover_path
-        elif self.cover_tempfiles['front'] == MusicBrainzRelease.NEED_UPDATE_COVER:
+        elif self.cover_tempfiles["front"] == MusicBrainzRelease.NEED_UPDATE_COVER:
             raise ValueError("Covers have not been downloaded yet; run update_covers()")
-        return self.cover_tempfiles['front'].name
+        return self.cover_tempfiles["front"].name
 
     @GObject.Property(type=str)
     def back_cover_path(self):
-        if not self.cover_tempfiles['back']:
-            return ''
-        elif self.cover_tempfiles['back'] == MusicBrainzRelease.NEED_UPDATE_COVER:
+        if not self.cover_tempfiles["back"]:
+            return ""
+        elif self.cover_tempfiles["back"] == MusicBrainzRelease.NEED_UPDATE_COVER:
             raise ValueError("Covers have not been downloaded yet; run update_covers()")
-        return self.cover_tempfiles['back'].name
+        return self.cover_tempfiles["back"].name
 
     def update_thumbnail(self):
         """Downloads the thumbnail for the release from coverartarchive.org"""
-        url = f'https://coverartarchive.org/release/{self.release_id}/front-250'
+        url = f"https://coverartarchive.org/release/{self.release_id}/front-250"
         if url in MusicBrainzRelease.cover_cache:
-            self.cover_tempfiles['thumbnail'] = MusicBrainzRelease.cover_cache[url]
-            self.notify('thumbnail-path')
+            self.cover_tempfiles["thumbnail"] = MusicBrainzRelease.cover_cache[url]
+            self.notify("thumbnail-path")
             return
 
-        if not self.full_data['cover-art-archive']['front']:
+        if not self.full_data["cover-art-archive"]["front"]:
             MusicBrainzRelease.cover_cache[url] = None
             # Try to get thumbnail for group instead
             self.group.update_thumbnail()
@@ -494,61 +551,66 @@ class MusicBrainzRelease(GObject.Object):
             return
 
         cover_extension = mimetypes.guess_extension(magic.from_buffer(data, mime=True))
-        self.cover_tempfiles['thumbnail'] = tempfile.NamedTemporaryFile(
+        self.cover_tempfiles["thumbnail"] = tempfile.NamedTemporaryFile(
             suffix=cover_extension
         )
-        self.cover_tempfiles['thumbnail'].write(data)
-        self.cover_tempfiles['thumbnail'].flush()
-        MusicBrainzRelease.cover_cache[url] = self.cover_tempfiles['thumbnail']
-        self.notify('thumbnail-path')
+        self.cover_tempfiles["thumbnail"].write(data)
+        self.cover_tempfiles["thumbnail"].flush()
+        MusicBrainzRelease.cover_cache[url] = self.cover_tempfiles["thumbnail"]
+        self.notify("thumbnail-path")
 
     def update_covers(self, if_needed=False):
         """Downloads the covers for the release from coverartarchive.org"""
-        if if_needed and self.cover_tempfiles['front'] != MusicBrainzRelease.NEED_UPDATE_COVER:
+        if (
+            if_needed
+            and self.cover_tempfiles["front"] != MusicBrainzRelease.NEED_UPDATE_COVER
+        ):
             return
 
         self.group.update_covers(if_needed)
 
-        if config.get_enum('musicbrainz-cover-size') == 0:
-            self.cover_tempfiles['front'] = ''
-            self.cover_tempfiles['back'] = ''
-            self.notify('front-cover-path')
-            self.notify('back-cover-path')
+        if config.get_enum("musicbrainz-cover-size") == 0:
+            self.cover_tempfiles["front"] = ""
+            self.cover_tempfiles["back"] = ""
+            self.notify("front-cover-path")
+            self.notify("back-cover-path")
             return
 
-        for cover in ('front', 'back'):
-            url = f'https://coverartarchive.org/release/{self.release_id}/{cover}'
+        for cover in ("front", "back"):
+            url = f"https://coverartarchive.org/release/{self.release_id}/{cover}"
 
-            if not self.full_data['cover-art-archive'][cover]:
+            if not self.full_data["cover-art-archive"][cover]:
                 MusicBrainzRelease.cover_cache[url] = None
-                self.cover_tempfiles[cover] = ''
-                self.notify(f'{cover}-cover-path')
+                self.cover_tempfiles[cover] = ""
+                self.notify(f"{cover}-cover-path")
                 continue
 
-            if config.get_enum('musicbrainz-cover-size') in (250, 500, 1200):
-                url += '-' + str(config.get_enum('musicbrainz-cover-size'))
+            if config.get_enum("musicbrainz-cover-size") in (250, 500, 1200):
+                url += "-" + str(config.get_enum("musicbrainz-cover-size"))
 
             if url in MusicBrainzRelease.cover_cache:
                 if MusicBrainzRelease.cover_cache[url]:
                     self.cover_tempfiles[cover] = MusicBrainzRelease.cover_cache[url]
                 else:
-                    self.cover_tempfiles[cover] = ''
-                self.notify(f'{cover}-cover-path')
+                    self.cover_tempfiles[cover] = ""
+                self.notify(f"{cover}-cover-path")
                 continue
 
             data = make_request(url, raw=True)
             if not data:
-                self.cover_tempfiles[cover] = ''
+                self.cover_tempfiles[cover] = ""
                 continue
 
-            cover_extension = mimetypes.guess_extension(magic.from_buffer(data, mime=True))
+            cover_extension = mimetypes.guess_extension(
+                magic.from_buffer(data, mime=True)
+            )
             self.cover_tempfiles[cover] = tempfile.NamedTemporaryFile(
                 suffix=cover_extension
             )
             self.cover_tempfiles[cover].write(data)
             self.cover_tempfiles[cover].flush()
             MusicBrainzRelease.cover_cache[url] = self.cover_tempfiles[cover]
-            self.notify(f'{cover}-cover-path')
+            self.notify(f"{cover}-cover-path")
 
     @classmethod
     def clear_tempfiles(cls):
@@ -560,7 +622,7 @@ class MusicBrainzRelease(GObject.Object):
                 pass
 
     def __str__(self):
-        return f'MusicBrainzRelease {self.release_id} ({self.title} - {self.artist})'
+        return f"MusicBrainzRelease {self.release_id} ({self.title} - {self.artist})"
 
     def __repr__(self):
         return self.__str__()
@@ -576,7 +638,8 @@ class MusicBrainzRelease(GObject.Object):
 
 class MusicBrainzReleaseGroup(GObject.Object):
     """A container for release group information, as found in the release query."""
-    __gtype_name__ = 'MusicBrainzReleaseGroup'
+
+    __gtype_name__ = "MusicBrainzReleaseGroup"
 
     NO_COVER = -2
 
@@ -588,18 +651,18 @@ class MusicBrainzReleaseGroup(GObject.Object):
         super().__init__()
         self._releases = None
         self.cover_tempfiles = {
-            'thumbnail': None,
-            'front': MusicBrainzRelease.NEED_UPDATE_COVER,
+            "thumbnail": None,
+            "front": MusicBrainzRelease.NEED_UPDATE_COVER,
         }
 
         if relgroup_data:
-            groupid = relgroup_data['id']
+            groupid = relgroup_data["id"]
         elif from_id:
             groupid = from_id
 
         if groupid not in MusicBrainzReleaseGroup.full_data_cache:
             MusicBrainzReleaseGroup.full_data_cache[groupid] = make_request(
-                build_url('release-group', groupid, inc=['releases'])
+                build_url("release-group", groupid, inc=["releases"])
             )
 
         if not MusicBrainzReleaseGroup.full_data_cache[groupid]:
@@ -610,116 +673,123 @@ class MusicBrainzReleaseGroup(GObject.Object):
         if groupid not in MusicBrainzReleaseGroup.obj_cache:
             MusicBrainzReleaseGroup.obj_cache[groupid] = self
 
-    @classmethod
-    def setup_from_id(cls, id):
-        if id in cls.obj_cache:
-            return cls.obj_cache[id]
-        return cls.__init__({}, from_id=id)
+    @staticmethod
+    def setup_from_id(id):
+        if id in MusicBrainzReleaseGroup.obj_cache:
+            return MusicBrainzReleaseGroup.obj_cache[id]
+        return MusicBrainzReleaseGroup({}, from_id=id)
 
     @GObject.Property(type=str)
     def relgroup_id(self):
-        return self.mb_data['id']
+        return self.mb_data["id"]
 
     @GObject.Property(type=str)
     def primary_type(self):
         try:
-            return self.mb_data['primary-type'].lower()
+            return self.mb_data["primary-type"].lower()
         except AttributeError:
-            return 'other'
+            return "other"
 
     @GObject.Property(type=str)
     def secondary_types(self):
         try:
-            return [t.lower() for t in self.mb_data['secondary-types']]
+            return [t.lower() for t in self.mb_data["secondary-types"]]
         except AttributeError:
             return []
 
     @GObject.Property
     def release_ids(self):
-        return [r['id'] for r in self.mb_data['releases']]
+        return [r["id"] for r in self.mb_data["releases"]]
 
     @GObject.Property
     def releases(self):
         if not self._releases:
-            self._releases = [MusicBrainzRelease.setup_from_id(id) for id in self.release_ids]
+            self._releases = [
+                MusicBrainzRelease.setup_from_id(id) for id in self.release_ids
+            ]
         return self._releases
 
     @GObject.Property(type=str)
     def thumbnail_path(self):
-        if not self.cover_tempfiles['thumbnail']:
+        if not self.cover_tempfiles["thumbnail"]:
             self.update_thumbnail()
-        if self.cover_tempfiles['thumbnail'] == MusicBrainzReleaseGroup.NO_COVER:
-            return ''
-        return self.cover_tempfiles['thumbnail'].name
+        if self.cover_tempfiles["thumbnail"] == MusicBrainzReleaseGroup.NO_COVER:
+            return ""
+        return self.cover_tempfiles["thumbnail"].name
 
     @GObject.Property(type=str)
     def front_cover_path(self):
-        if not self.cover_tempfiles['front'] or \
-                self.cover_tempfiles['front'] == MusicBrainzReleaseGroup.NO_COVER:
-            return ''
-        elif self.cover_tempfiles['front'] == MusicBrainzRelease.NEED_UPDATE_COVER:
+        if (
+            not self.cover_tempfiles["front"]
+            or self.cover_tempfiles["front"] == MusicBrainzReleaseGroup.NO_COVER
+        ):
+            return ""
+        elif self.cover_tempfiles["front"] == MusicBrainzRelease.NEED_UPDATE_COVER:
             raise ValueError("Covers have not been downloaded yet; run update_covers()")
-        return self.cover_tempfiles['front'].name
+        return self.cover_tempfiles["front"].name
 
     def update_thumbnail(self):
         """Downloads the thumbnail for the release from coverartarchive.org"""
-        url = f'https://coverartarchive.org/release-group/{self.relgroup_id}/front-250'
+        url = f"https://coverartarchive.org/release-group/{self.relgroup_id}/front-250"
         if url in MusicBrainzRelease.cover_cache:
-            self.cover_tempfiles['thumbnail'] = MusicBrainzRelease.cover_cache[url]
-            self.notify('thumbnail-path')
+            self.cover_tempfiles["thumbnail"] = MusicBrainzRelease.cover_cache[url]
+            self.notify("thumbnail-path")
             return
 
         data = make_request(url, raw=True)
         if not data:
-            self.cover_tempfiles['thumbnail'] = MusicBrainzReleaseGroup.NO_COVER
-            self.notify('thumbnail-path')
+            self.cover_tempfiles["thumbnail"] = MusicBrainzReleaseGroup.NO_COVER
+            self.notify("thumbnail-path")
             return
 
         cover_extension = mimetypes.guess_extension(magic.from_buffer(data, mime=True))
-        self.cover_tempfiles['thumbnail'] = tempfile.NamedTemporaryFile(
+        self.cover_tempfiles["thumbnail"] = tempfile.NamedTemporaryFile(
             suffix=cover_extension
         )
-        self.cover_tempfiles['thumbnail'].write(data)
-        self.cover_tempfiles['thumbnail'].flush()
-        MusicBrainzRelease.cover_cache[url] = self.cover_tempfiles['thumbnail']
-        self.notify('thumbnail-path')
+        self.cover_tempfiles["thumbnail"].write(data)
+        self.cover_tempfiles["thumbnail"].flush()
+        MusicBrainzRelease.cover_cache[url] = self.cover_tempfiles["thumbnail"]
+        self.notify("thumbnail-path")
 
     def update_covers(self, if_needed=False):
         """Downloads the covers for the release from coverartarchive.org"""
-        if if_needed and self.cover_tempfiles['front'] != MusicBrainzRelease.NEED_UPDATE_COVER:
+        if (
+            if_needed
+            and self.cover_tempfiles["front"] != MusicBrainzRelease.NEED_UPDATE_COVER
+        ):
             return
 
-        if config.get_enum('musicbrainz-cover-size') == 0:
-            self.cover_tempfiles['front'] = ''
+        if config.get_enum("musicbrainz-cover-size") == 0:
+            self.cover_tempfiles["front"] = ""
             return
 
-        url = f'https://coverartarchive.org/release-group/{self.relgroup_id}/front'
+        url = f"https://coverartarchive.org/release-group/{self.relgroup_id}/front"
 
-        if config.get_enum('musicbrainz-cover-size') in (250, 500, 1200):
-            url += '-' + str(config.get_enum('musicbrainz-cover-size'))
+        if config.get_enum("musicbrainz-cover-size") in (250, 500, 1200):
+            url += "-" + str(config.get_enum("musicbrainz-cover-size"))
 
         if url in MusicBrainzRelease.cover_cache:
             if MusicBrainzRelease.cover_cache[url]:
-                self.cover_tempfiles['front'] = MusicBrainzRelease.cover_cache[url]
+                self.cover_tempfiles["front"] = MusicBrainzRelease.cover_cache[url]
             else:
-                self.cover_tempfiles['front'] = ''
-            self.notify('front-cover-path')
+                self.cover_tempfiles["front"] = ""
+            self.notify("front-cover-path")
             return
 
         data = make_request(url, raw=True)
         if not data:
-            self.cover_tempfiles['front'] = MusicBrainzReleaseGroup.NO_COVER
-            self.notify('front-cover-path')
+            self.cover_tempfiles["front"] = MusicBrainzReleaseGroup.NO_COVER
+            self.notify("front-cover-path")
             return
 
         cover_extension = mimetypes.guess_extension(magic.from_buffer(data, mime=True))
-        self.cover_tempfiles['front'] = tempfile.NamedTemporaryFile(
+        self.cover_tempfiles["front"] = tempfile.NamedTemporaryFile(
             suffix=cover_extension
         )
-        self.cover_tempfiles['front'].write(data)
-        self.cover_tempfiles['front'].flush()
-        MusicBrainzRelease.cover_cache[url] = self.cover_tempfiles['front']
-        self.notify('front-cover-path')
+        self.cover_tempfiles["front"].write(data)
+        self.cover_tempfiles["front"].flush()
+        MusicBrainzRelease.cover_cache[url] = self.cover_tempfiles["front"]
+        self.notify("front-cover-path")
 
     @classmethod
     def clear_tempfiles(cls):
@@ -738,6 +808,7 @@ class MusicBrainzReleaseGroup(GObject.Object):
     def __hash__(self):
         return id(self)
 
+
 def acoustid_identify_file(file):
     """
     Uses AcoustID and Chromaprint to identify a track's data.
@@ -747,21 +818,25 @@ def acoustid_identify_file(file):
     """
     try:
         results = acoustid.match(ACOUSTID_API_KEY, file.path, parse=False)
-        if 'results' not in results or not results['results']:
+        if "results" not in results or not results["results"]:
             return (0.0, None)
     except:
+        print(
+            f"Error while getting AcoustID match for {os.path.basename(file.path)} ({file.id}):"
+        )
         traceback.print_exc()
+        print("Continuing without match. (This is not a fatal error!)")
         return (0.0, None)
 
-    acoustid_data = results['results'][0]
+    acoustid_data = results["results"][0]
 
-    if acoustid_data['score'] * 100 < config['acoustid-confidence-treshold']:
+    if acoustid_data["score"] * 100 < config["acoustid-confidence-treshold"]:
         return (0.0, None)
 
-    if 'recordings' in acoustid_data:
-        musicbrainz_id = acoustid_data['recordings'][0]['id']
+    if "recordings" in acoustid_data:
+        musicbrainz_id = acoustid_data["recordings"][0]["id"]
         rec = MusicBrainzRecording(musicbrainz_id, file=file)
 
-        return (acoustid_data['score'], rec)
+        return (acoustid_data["score"], rec)
 
     return (0.0, None)
